/******************************************************************************/
/*                                                                            */
/*  (c) Copyright Hewlett-Packard Company, 1993, Fort Collins, Colorado       */
/*                                                                            */
/*  (C) COPYRIGHT International Business Machines Corp. 1993                  */
/*                                                                            */
/*                            All Rights Reserved                             */
/*                                                                            */
/*  Permission to use, copy, modify, and distribute this software and its     */
/*  documentation for any purpose and without fee is hereby granted,          */
/*  provided that the above copyright notices appear in all copies and that   */
/*  both the copyright notices and this permission notice appear in           */
/*  supporting documentation, and that the name of Hewlett-Packard not be     */
/*  used in advertising or publicity pertaining to distribution of the        */
/*  software without specific, written prior permission.                      */
/*                                                                            */
/*  HEWLETT-PACKARD MAKES NO WARRANTY OF ANY KIND WITH REGARD TO THIS         */
/*  SOFTWARE, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/*  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Hewlett-Packard    */
/*  shall not be liable for errors contained herein or direct, indirect,      */
/*  special, incidental or consequential damages in connection with the       */
/*  furnishing, performance or use of this software.                          */
/*                                                                            */
/*  INTERNATIONAL BUSINESS MACHINES CORPORATION PROVIDES THIS MATERIAL        */
/*  "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,        */
/*  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY  */
/*  AND FITNESS FOR A PARTICULAR PURPOSE.                                     */
/*                                                                            */
/******************************************************************************/

/*
 * tmlut_edit
 *
 * This test checks out the various aspects of editing a lookup table. It
 * tests out entry setting, entry deletion, and entry inquiry.
 *
 * The basic mechanisms for all LUTs are the same. So, the mechanisms of 
 * deletion and insertion are first checked out on a TMSampling LUT. The
 * remainder of this test just sets a few entries for each type of LUT and
 * then inquires the LUTs. These tests are designed to check the set/realized
 * semantics of entry setting, check the 2 entry inquiry entry points, and check
 * that all entry fields are set correctly and returned on inquiry. 
 * 
 * Taken from Ken Tidwell's 5.1 LUT test
 * Sharad Naik
 */

/*
 * Basic set of include files for tests
 */
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/PEX5/PEXlib.h>
#include <X11/PEX5/PEXExtlib.h>
#include <misc.h>

#define NUM_LUTS 4
#define NUM_ENTRIES (NUM_LUTS + 1)
#define TMBinding          1
#define TMCoordSource      2
#define TMComposition      3
#define TMSampling         4

Window local_window;

void lut_test(Display *display,
	      Window   window)
{
    /*************************************************************************
     * Test specific data
     ************************************************************************/
    PEXLookupTable luts[NUM_ENTRIES];     /* 4 lut types, 0'th entry ignored */
    PEXPointer entries [NUM_ENTRIES];     /* 4 lut types, 0'th entry ignored */

    int i, j;
    int table_type, entry_type;
    Status status;
    unsigned long return_count;
    PEXTableIndex *return_indices;
    unsigned long num_fonts;
    XID tmid;
     

    PEXExtTMBindingEntry        *p_TMBinding, *p_TMBinding_tmp;
    PEXExtTMCoordSourceEntry    *p_TMCoordSource, *p_TMCoordSource_tmp;
    PEXExtTMCompositionEntry    *p_TMComposition, *p_TMComposition_tmp;
    PEXExtTMSamplingEntry       *p_TMSampling, *p_TMSampling_tmp;

    /**********************************************************************
     * Test begins here
     *********************************************************************/

    /*
     * Create a full set of LUTS to play with during this test.
     * No error checking here since other tests will check this out.
     */

    for (i=1; i<NUM_ENTRIES; i++)
    {
	luts[i] = PEXCreateLookupTable (display, window, i);
    }

    /*********
     * Step 1	Set entries sparsely in table and check them out
     ********/

    /*
     * Allocate a buffer for setting entries
     */
    p_TMSampling = (PEXExtTMSamplingEntry *) 
                    malloc(3*sizeof(PEXExtTMSamplingEntry));
    p_TMSampling_tmp = p_TMSampling;
    
    /*
     * Fill out buffer
     */
    p_TMSampling_tmp->minification_method = PEXExtTMTexelSampleSingleBase;
    p_TMSampling_tmp->magnification_method = PEXExtTMTexelSampleSingleBase;
    p_TMSampling_tmp->t0_boundary_condition = PEXExtTMBoundaryCondWrap;
    p_TMSampling_tmp->t1_boundary_condition = PEXExtTMBoundaryCondWrap;
    p_TMSampling_tmp->boundary_clamp_color_source = 
                                         PEXExtTMClampColorSourceBoundary;
/*
    p_TMSampling_tmp->clamp_color.type = PEXColorTypeRGB;
    p_TMSampling_tmp->clamp_color.value.rgb.red = 1.0;
    p_TMSampling_tmp->clamp_color.value.rgb.green = 1.0;
    p_TMSampling_tmp->clamp_color.value.rgb.blue = 1.0;
*/
    p_TMSampling_tmp->depth_sampling_bias_hint = 1.0;
    p_TMSampling_tmp->t0_frequency_hint = 1.0;
    p_TMSampling_tmp->t1_frequency_hint = 1.0;
    p_TMSampling_tmp->t2_frequency_hint = 1.0;

    p_TMSampling_tmp++;
    p_TMSampling_tmp->minification_method = PEXExtTMTexelSampleSingleInMipmap;
    p_TMSampling_tmp->magnification_method = PEXExtTMTexelSampleSingleBase;
    p_TMSampling_tmp->t0_boundary_condition = 
                      PEXExtTMBoundaryCondClampAbsolute;
    p_TMSampling_tmp->t1_boundary_condition = 
                      PEXExtTMBoundaryCondClampAbsolute;
    p_TMSampling_tmp->boundary_clamp_color_source =
                                         PEXExtTMClampColorSourceBoundary;
/*
    p_TMSampling_tmp->clamp_color.type = PEXColorTypeRGB;
    p_TMSampling_tmp->clamp_color.value.rgb.red = 1.0;
    p_TMSampling_tmp->clamp_color.value.rgb.green = 1.0;
    p_TMSampling_tmp->clamp_color.value.rgb.blue = 1.0;
*/
    p_TMSampling_tmp->depth_sampling_bias_hint = 2.0;
    p_TMSampling_tmp->t0_frequency_hint = 0.5;
    p_TMSampling_tmp->t1_frequency_hint = 0.5;
    p_TMSampling_tmp->t2_frequency_hint = 0.8;

    p_TMSampling_tmp++;
    p_TMSampling_tmp->minification_method = 
                           PEXExtTMTexelSampleLinearInMipmap;
    p_TMSampling_tmp->magnification_method = PEXExtTMTexelSampleLinearBase;
    p_TMSampling_tmp->t0_boundary_condition = 
                      PEXExtTMBoundaryCondClampColor;
    p_TMSampling_tmp->t1_boundary_condition = 
                      PEXExtTMBoundaryCondClampColor;
    p_TMSampling_tmp->boundary_clamp_color_source =
                                         PEXExtTMClampColorSourceExplicit;
    p_TMSampling_tmp->clamp_color.type = PEXColorTypeRGB;
    p_TMSampling_tmp->clamp_color.value.rgb.red = 1.0;
    p_TMSampling_tmp->clamp_color.value.rgb.green = 1.0;
    p_TMSampling_tmp->clamp_color.value.rgb.blue = 1.0;
    p_TMSampling_tmp->depth_sampling_bias_hint = 1.5;
    p_TMSampling_tmp->t0_frequency_hint = 0.8;
    p_TMSampling_tmp->t1_frequency_hint = 0.8;
    p_TMSampling_tmp->t2_frequency_hint = 0.8;
    
    /*
     * Set entries sparsely
     */
    PEXExtSetTableEntries (display, luts[TMSampling], 511, 3, 
			   PEXExtLUTTMSampling, p_TMSampling);
     
    PEXExtSetTableEntries (display, luts[TMSampling], 619, 2, 
			   PEXExtLUTTMSampling, p_TMSampling);

    PEXExtSetTableEntries (display, luts[TMSampling], 65530, 2, 
			   PEXExtLUTTMSampling, p_TMSampling);

    /*
     * See if defined indices are what we expect.
     */
    status = PEXGetDefinedIndices (display, luts[TMSampling], 
				   &return_count, &return_indices);

    if (! ( (status) &&
	    (return_count == 8) &&
	    (return_indices[0] == 0) &&
	    (return_indices[1] == 511) &&
	    (return_indices[2] == 512) &&
	    (return_indices[3] == 513) &&
	    (return_indices[4] == 619) &&
	    (return_indices[5] == 620) &&
	    (return_indices[6] == 65530) &&
	    (return_indices[7] == 65531)
	    ))
    {
	_ibmpex_stderr_print
	     ( "ERROR: sparse TMSampling not set up as expected\n");
    }

    XFree (return_indices);

    /*
     * Delete a couple of entries. Are defined indices still as expected?
     */
    PEXDeleteTableEntries (display, luts[TMSampling], 511, 2);

    status = PEXGetDefinedIndices (display, luts[TMSampling], 
				   &return_count, &return_indices);


    if (! ( (status) &&
	    (return_count == 6) &&
	    (return_indices[0] == 0) &&
	    (return_indices[1] == 513) &&
	    (return_indices[2] == 619) &&
	    (return_indices[3] == 620) &&
	    (return_indices[4] == 65530) &&
	    (return_indices[5] == 65531)
	    ))
    {
	_ibmpex_stderr_print
	     ( "ERROR: TMSampling not set up as expected following delete\n");
    }

    XFree (return_indices);


    /*
     * Get back some of the entries and see if they are correct. We try to
     * get our deleted entries, some never set entries and some of the defined
     * entries. We also check the set vs. realized values in a few cases.
     */

    /*
     * Get a real entry
     */

    p_TMSampling_tmp = (PEXExtTMSamplingEntry*) PEXExtGetTableEntry (display, 
					 luts[TMSampling], 0, 
		                         PEXRealizedValue, &entry_type, 
					 &table_type);

    if (!( (entry_type == PEXDefinedEntry) &&
	   (table_type == PEXExtLUTTMSampling) &&
    	   (p_TMSampling_tmp->minification_method ==
                                 PEXExtTMTexelSampleSingleBase) &&
    	   (p_TMSampling_tmp->magnification_method == 
                                 PEXExtTMTexelSampleSingleBase) &&
           (p_TMSampling_tmp->t0_boundary_condition ==
                                 PEXExtTMBoundaryCondClampAbsolute) &&
           (p_TMSampling_tmp->t1_boundary_condition ==
                                 PEXExtTMBoundaryCondClampAbsolute) &&
           (p_TMSampling_tmp->depth_sampling_bias_hint = 0.0) &&
           (p_TMSampling_tmp->t0_frequency_hint = 1.0) &&
           (p_TMSampling_tmp->t1_frequency_hint = 1.0) &&
           (p_TMSampling_tmp->t2_frequency_hint = 1.0)  ))
    {
	_ibmpex_stderr_print( "ERROR: TMSampling Get Entry 0 failed\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMSampling, 1, p_TMSampling_tmp);


    /*
     * Get a deleted entry
     */
    p_TMSampling_tmp = (PEXExtTMSamplingEntry*)PEXExtGetTableEntry (display, 
					 luts[TMSampling], 511, 
		                         PEXRealizedValue, &entry_type, 
					 &table_type);

    if (!( (entry_type == PEXDefaultEntry) &&
	   (table_type == PEXExtLUTTMSampling) &&
           (p_TMSampling_tmp->minification_method ==
                                 PEXExtTMTexelSampleSingleBase) &&
           (p_TMSampling_tmp->magnification_method ==
                                 PEXExtTMTexelSampleSingleBase) &&
           (p_TMSampling_tmp->t0_boundary_condition ==
                                 PEXExtTMBoundaryCondClampAbsolute) &&
           (p_TMSampling_tmp->t1_boundary_condition ==
                                 PEXExtTMBoundaryCondClampAbsolute) &&
           (p_TMSampling_tmp->depth_sampling_bias_hint = 0.0) &&
           (p_TMSampling_tmp->t0_frequency_hint = 1.0) &&
           (p_TMSampling_tmp->t1_frequency_hint = 1.0) &&
           (p_TMSampling_tmp->t2_frequency_hint = 1.0)  ))

    {
	_ibmpex_stderr_print( "ERROR: TMSampling Get Entry 2 failed\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMSampling, 1, p_TMSampling_tmp);

    /*
     * Get a "virgin" entry
     */
    p_TMSampling_tmp = (PEXExtTMSamplingEntry*) PEXExtGetTableEntry (display, 
					 luts[TMSampling], 900, 
		                         PEXRealizedValue, &entry_type, 
					 &table_type);

    if (!( (entry_type == PEXDefaultEntry) &&
	   (table_type == PEXExtLUTTMSampling) &&
           (p_TMSampling_tmp->minification_method ==
                                 PEXExtTMTexelSampleSingleBase) &&
           (p_TMSampling_tmp->magnification_method ==
                                 PEXExtTMTexelSampleSingleBase) &&
           (p_TMSampling_tmp->t0_boundary_condition ==
                                 PEXExtTMBoundaryCondClampAbsolute) &&
           (p_TMSampling_tmp->t1_boundary_condition ==
                                 PEXExtTMBoundaryCondClampAbsolute) &&
           (p_TMSampling_tmp->depth_sampling_bias_hint = 0.0) &&
           (p_TMSampling_tmp->t0_frequency_hint = 1.0) &&
           (p_TMSampling_tmp->t1_frequency_hint = 1.0) &&
           (p_TMSampling_tmp->t2_frequency_hint = 1.0)  ))
    {
	_ibmpex_stderr_print( "ERROR: TMSampling Get Entry 3 failed\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMSampling, 1, p_TMSampling_tmp);

    /*
     * Get set values from an entry
     */
    p_TMSampling_tmp = (PEXExtTMSamplingEntry*) PEXExtGetTableEntry (display, 
					 luts[TMSampling], 513, 
		                         PEXSetValue, &entry_type, 
					 &table_type);

    if (!( (entry_type == PEXDefinedEntry) &&
	   (table_type == PEXExtLUTTMSampling) &&
           (p_TMSampling_tmp->minification_method ==
                                 PEXExtTMTexelSampleLinearInMipmap) &&
           (p_TMSampling_tmp->magnification_method ==
                                 PEXExtTMTexelSampleLinearBase) &&
           (p_TMSampling_tmp->t0_boundary_condition ==
                                 PEXExtTMBoundaryCondClampColor) &&
           (p_TMSampling_tmp->t1_boundary_condition ==
                                 PEXExtTMBoundaryCondClampColor) &&
           (p_TMSampling_tmp->boundary_clamp_color_source ==
                                 PEXExtTMClampColorSourceExplicit) &&
           (p_TMSampling_tmp->clamp_color.type == PEXColorTypeRGB) &&
           (p_TMSampling_tmp->clamp_color.value.rgb.red == 1.0) &&
           (p_TMSampling_tmp->clamp_color.value.rgb.green == 1.0) &&
           (p_TMSampling_tmp->clamp_color.value.rgb.blue == 1.0) &&
           (p_TMSampling_tmp->depth_sampling_bias_hint == 1.5) &&
           (p_TMSampling_tmp->t0_frequency_hint == 0.8) &&
           (p_TMSampling_tmp->t1_frequency_hint == 0.8) &&
           (p_TMSampling_tmp->t2_frequency_hint == 0.8) ))
    {
	_ibmpex_stderr_print( "ERROR: TMSampling Get Entry 4 failed\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMSampling, 1, p_TMSampling_tmp);

    /*
     * Get some entries. Some defined and some not. Check 'em out.
     */

    status = PEXExtGetTableEntries (display, luts[TMSampling], 618, 4,  
	                         PEXSetValue, &table_type, 
				 (PEXPointer *) &p_TMSampling);

    if (! status)
    {
	_ibmpex_stderr_print
	     ( "ERROR: Final Get MSampling Table Entries failed\n");
    }

    if (! (
           (p_TMSampling[0].minification_method ==
                                 PEXExtTMTexelSampleSingleBase) &&
           (p_TMSampling[0].magnification_method ==
                                 PEXExtTMTexelSampleSingleBase) &&
           (p_TMSampling[0].t0_boundary_condition ==
                                 PEXExtTMBoundaryCondClampAbsolute) &&
           (p_TMSampling[0].t1_boundary_condition ==
                                 PEXExtTMBoundaryCondClampAbsolute) &&
           (p_TMSampling[0].depth_sampling_bias_hint = 0.0) &&
           (p_TMSampling[0].t0_frequency_hint = 1.0) &&
           (p_TMSampling[0].t1_frequency_hint = 1.0) &&
           (p_TMSampling[0].t2_frequency_hint = 1.0)  ))
    {
	_ibmpex_stderr_print
	     ("ERROR: Final TMSampling Table Entry 1 is incorrect\n");
    }


    if (! (
           (p_TMSampling[1].minification_method ==
                                PEXExtTMTexelSampleSingleBase) &&
           (p_TMSampling[1].magnification_method ==
                                 PEXExtTMTexelSampleSingleBase) &&
           (p_TMSampling[1].t0_boundary_condition ==
                                 PEXExtTMBoundaryCondWrap) &&
           (p_TMSampling[1].t1_boundary_condition ==
                                 PEXExtTMBoundaryCondWrap) &&
           (p_TMSampling[1].boundary_clamp_color_source ==
                                 PEXExtTMClampColorSourceBoundary) &&
           (p_TMSampling[1].depth_sampling_bias_hint == 1.0) &&
           (p_TMSampling[1].t0_frequency_hint == 1.0) &&
           (p_TMSampling[1].t1_frequency_hint == 1.0) &&
           (p_TMSampling[1].t2_frequency_hint == 1.0) ))
    {
        _ibmpex_stderr_print
             ("ERROR: Final TMSampling Table Entry 2 is incorrect\n");
    }

    if (! (
           (p_TMSampling[2].minification_method ==
                                PEXExtTMTexelSampleSingleInMipmap) &&
           (p_TMSampling[2].magnification_method ==
                                 PEXExtTMTexelSampleSingleBase) &&
           (p_TMSampling[2].t0_boundary_condition ==
                                 PEXExtTMBoundaryCondClampAbsolute) &&
           (p_TMSampling[2].t1_boundary_condition ==
                                 PEXExtTMBoundaryCondClampAbsolute) &&
           (p_TMSampling[2].boundary_clamp_color_source ==
                                 PEXExtTMClampColorSourceBoundary) &&
           (p_TMSampling[2].depth_sampling_bias_hint == 2.0) &&
           (p_TMSampling[2].t0_frequency_hint = 0.5) &&
           (p_TMSampling[2].t1_frequency_hint = 0.5) &&
           (p_TMSampling[2].t2_frequency_hint = 0.8) ))
    {
	_ibmpex_stderr_print
	    ("ERROR: Final TMSampling Table Entry 3 is incorrect\n");
    }


    if (! (
           (p_TMSampling[3].minification_method ==
                                 PEXExtTMTexelSampleSingleBase) &&
           (p_TMSampling[3].magnification_method ==
                                 PEXExtTMTexelSampleSingleBase) &&
           (p_TMSampling[3].t0_boundary_condition ==
                                 PEXExtTMBoundaryCondClampAbsolute) &&
           (p_TMSampling[3].t1_boundary_condition ==
                                 PEXExtTMBoundaryCondClampAbsolute) &&
           (p_TMSampling[3].depth_sampling_bias_hint = 0.0) &&
           (p_TMSampling[3].t0_frequency_hint = 1.0) &&
           (p_TMSampling[3].t1_frequency_hint = 1.0) &&
           (p_TMSampling[3].t2_frequency_hint = 1.0)  ))
    {
	_ibmpex_stderr_print
	    ("ERROR: Final TMSampling Table Entry 4 is incorrect\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMSampling, 4, p_TMSampling);
 
    /*
     * Check realized value
     */
    status = PEXExtGetTableEntries (display, luts[TMSampling], 620, 1,  
	                         PEXRealizedValue, &table_type, 
				 (PEXPointer *) &p_TMSampling);

    if (! (
           (p_TMSampling[0].minification_method ==
                                PEXExtTMTexelSampleSingleInMipmap) &&
           (p_TMSampling[0].magnification_method ==
                                 PEXExtTMTexelSampleSingleBase) &&
           (p_TMSampling[0].t0_boundary_condition ==
                                 PEXExtTMBoundaryCondClampAbsolute) &&
           (p_TMSampling[0].t1_boundary_condition ==
                                 PEXExtTMBoundaryCondClampAbsolute) &&
           (p_TMSampling[0].boundary_clamp_color_source ==
                                 PEXExtTMClampColorSourceBoundary) &&
           (p_TMSampling[0].depth_sampling_bias_hint == 2.0) &&
           (p_TMSampling[0].t0_frequency_hint = 0.5) &&
           (p_TMSampling[0].t1_frequency_hint = 0.5) &&
           (p_TMSampling[0].t2_frequency_hint = 0.8) ))
    {
	_ibmpex_stderr_print
	    ("ERROR: Realized TMSampling Table Entry is incorrect\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMSampling, 1, p_TMSampling);
    
    /*
     * At this point we have checked out the deletion mechanism and the
     * Get Defined Indices mechanism. There is no need to do any more of
     * that. The remainder of this test will concentrate on making sure
     * that all fields of lut entries are set correctly and are inquired
     * correctly. We will also verify set/realized fields in the entries.
     * Since Get Entry and Get Entries use the same support routine to collect
     * the entry data, we do not exhaustive test each entry point. Rather,
     * we use both API routines to cover the interesting cases for entry
     * retrieval.
     */



    /*
     * Check out TMBinding
     */


    /*
     * Set a couple entries
     */
    p_TMBinding = (PEXExtTMBindingEntry * )
				malloc(2*sizeof(PEXExtTMBindingEntry));
    p_TMBinding_tmp = p_TMBinding;
    
    tmid = 10;
    p_TMBinding_tmp->tm_description_id = tmid;
    p_TMBinding_tmp->coord_source_index = 1;
    p_TMBinding_tmp->composition_index = 1;
    p_TMBinding_tmp->sampling_index = 1;

    p_TMBinding_tmp++;
    p_TMBinding_tmp->tm_description_id = tmid;
    p_TMBinding_tmp->coord_source_index = 1;
    p_TMBinding_tmp->composition_index = 2;
    p_TMBinding_tmp->sampling_index = 3;

    PEXExtSetTableEntries (display, luts[TMBinding], 511, 2, 
			PEXExtLUTTMBinding, p_TMBinding);

    free (p_TMBinding);
     
    /*
     * Get the realized values of a defined entry
     */
    p_TMBinding_tmp = (PEXExtTMBindingEntry*) PEXExtGetTableEntry(display, 
					 luts[TMBinding], 
					 512, PEXRealizedValue, &entry_type, 
					 &table_type);

    if (!((p_TMBinding_tmp->tm_description_id == tmid) &&
          (p_TMBinding_tmp->coord_source_index == 1) &&
          (p_TMBinding_tmp->composition_index == 1) &&
          (p_TMBinding_tmp->sampling_index == 1) &&
          (entry_type == PEXDefinedEntry) &&
          (table_type == PEXExtLUTTMBinding) ))
    {
	_ibmpex_stderr_print
	    ( "Get defined TMBinding Entry returned incorrect data\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMBinding, 1, p_TMBinding_tmp);

    /*
     * Get the set values of an undefined entry
     */
    p_TMBinding_tmp = (PEXExtTMBindingEntry*)PEXExtGetTableEntry(display, 
					luts[TMBinding], 
					 710, PEXSetValue, &entry_type, 
					 &table_type);

    if (!((p_TMBinding_tmp->coord_source_index == 0) &&
          (p_TMBinding_tmp->composition_index == 0) &&
          (p_TMBinding_tmp->sampling_index == 0) &&
	  (entry_type == PEXDefaultEntry) &&
	  (table_type == PEXExtLUTTMBinding) ))
    {
	_ibmpex_stderr_print
	    ( "Get undefined TMBinding Entry returned incorrect data\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMBinding, 1, p_TMBinding_tmp);
    
    /*
     * Get a couple entries at once
     */
    status = PEXExtGetTableEntries (display, luts[TMBinding], 510, 3,  
	                         PEXSetValue, &table_type, 
				 (PEXPointer *) &p_TMBinding);

    if (!((p_TMBinding_tmp[0].coord_source_index == 0) &&
          (p_TMBinding_tmp[0].composition_index == 0) &&
          (p_TMBinding_tmp[0].sampling_index == 0) &&
          (p_TMBinding_tmp[1].tm_description_id == tmid) &&
          (p_TMBinding_tmp[1].coord_source_index == 1) &&
          (p_TMBinding_tmp[1].composition_index == 1) &&
          (p_TMBinding_tmp[1].sampling_index == 1) &&
          (p_TMBinding_tmp[2].tm_description_id == tmid) &&
          (p_TMBinding_tmp[2].coord_source_index == 1) &&
          (p_TMBinding_tmp[2].composition_index == 2) &&
          (p_TMBinding_tmp[2].sampling_index == 3) ))
    {
	_ibmpex_stderr_print
	    ( "Get TMBinding Entries returned incorrect data\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMBinding, 3, p_TMBinding_tmp);



    /*
     * Check out TMCoordSource
     */


    /*
     * Set a couple entries
     */
    p_TMCoordSource = (PEXExtTMCoordSourceEntry *) 
				malloc(2*sizeof(PEXExtTMCoordSourceEntry));
    p_TMCoordSource_tmp = p_TMCoordSource;
    
    p_TMCoordSource_tmp->tm_source = PEXExtTMCoordSourceVertexNormal;

    p_TMCoordSource_tmp++;
    p_TMCoordSource_tmp->tm_source = PEXExtTMCoordSourceFloatData;
    p_TMCoordSource_tmp->fp_data_index = 4;

    PEXExtSetTableEntries (display, luts[TMCoordSource], 1023, 2, 
			PEXExtLUTTMCoordSource, p_TMCoordSource);

    free (p_TMCoordSource);
     
    /*
     * Get the realized values of a defined entry
     */
    p_TMCoordSource_tmp = (PEXExtTMCoordSourceEntry*)PEXExtGetTableEntry(display, 
					luts[TMCoordSource], 
					 1024, PEXRealizedValue, &entry_type, 
					 &table_type);

    if (!((p_TMCoordSource_tmp->tm_source ==
                               PEXExtTMCoordSourceFloatData) &&
                  (p_TMCoordSource_tmp->fp_data_index  == 4) &&
	          (entry_type == PEXDefinedEntry) &&
	          (table_type == PEXExtLUTTMCoordSource) ))
    {
	_ibmpex_stderr_print
	    ( "Get defined TMCoordSource Entry returned incorrect data\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMCoordSource, 1, p_TMCoordSource_tmp);

    /*
     * Get the set values of an undefined entry
     */
    p_TMCoordSource_tmp = (PEXExtTMCoordSourceEntry*)PEXExtGetTableEntry(display, 
					luts[TMCoordSource], 
					 10, PEXSetValue, &entry_type, 
					 &table_type);

    if (!((p_TMCoordSource_tmp->tm_source ==
                               PEXExtTMCoordSourceFloatData) &&
                  (p_TMCoordSource_tmp->fp_data_index  == 0) &&
	          (entry_type == PEXDefaultEntry) &&
                  (table_type == PEXExtLUTTMCoordSource) ))
    {
	_ibmpex_stderr_print
	    ( "Get undefined TMCoordSource Entry returned incorrect data\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMCoordSource, 1, p_TMCoordSource_tmp);
    
    /*
     * Get a couple entries at once
     */
    status = PEXExtGetTableEntries (display, luts[TMCoordSource], 1023, 3,  
	                         PEXSetValue, &table_type, 
				 (PEXPointer *) &p_TMCoordSource_tmp);

    if (!((table_type == PEXExtLUTTMCoordSource) &&
          (p_TMCoordSource_tmp[0].tm_source == 
                                  PEXExtTMCoordSourceVertexNormal) &&
          (p_TMCoordSource_tmp[1].tm_source ==
                                  PEXExtTMCoordSourceFloatData) &&
          (p_TMCoordSource_tmp[1].fp_data_index  == 4) &&
          (p_TMCoordSource_tmp[2].tm_source ==
                                  PEXExtTMCoordSourceFloatData) &&
          (p_TMCoordSource_tmp[2].fp_data_index  == 0) ))
    {
	_ibmpex_stderr_print
	    ( "Get TMCoordSource Entries returned incorrect data\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMCoordSource, 3, p_TMCoordSource_tmp);


    /*
     * Check out TMComposition 
     */


    /*
     * Set a couple entries
     */
    p_TMComposition = (PEXExtTMCompositionEntry *) 
				malloc(2*sizeof(PEXExtTMCompositionEntry));
    p_TMComposition_tmp = p_TMComposition;
    
    p_TMComposition_tmp->method = PEXExtTMCompositeDecalBackground;
    p_TMComposition_tmp->data.decal_bkgd_color.type = PEXColorTypeRGB;
    p_TMComposition_tmp->data.decal_bkgd_color.value.rgb.red = 1.0;
    p_TMComposition_tmp->data.decal_bkgd_color.value.rgb.green = 0.0;
    p_TMComposition_tmp->data.decal_bkgd_color.value.rgb.blue = 1.0;

    p_TMComposition_tmp++;
    p_TMComposition_tmp->method = PEXExtTMCompositeBlendEnvColor;
    p_TMComposition_tmp->data.blend_env.channel_number = 0;
    p_TMComposition_tmp->data.blend_env.color.type = PEXColorTypeRGB;
    p_TMComposition_tmp->data.blend_env.color.value.rgb.red = 1.0;
    p_TMComposition_tmp->data.blend_env.color.value.rgb.green = 1.0;
    p_TMComposition_tmp->data.blend_env.color.value.rgb.blue = 1.0;

    PEXExtSetTableEntries (display, luts[TMComposition], 255, 2, 
			PEXExtLUTTMComposition, p_TMComposition);

    free (p_TMComposition);
     
    /*
     * Get the realized values of a defined entry
     */
    p_TMComposition_tmp = (PEXExtTMCompositionEntry*)PEXExtGetTableEntry(display, 
					luts[TMComposition],
					 255, PEXRealizedValue, &entry_type, 
					 &table_type);

    if (!((entry_type == PEXDefinedEntry) &&
	  (table_type == PEXExtLUTTMComposition) &&
          (p_TMComposition_tmp->method == PEXExtTMCompositeDecalBackground) &&
          (p_TMComposition_tmp->data.decal_bkgd_color.value.rgb.red == 1.0) &&
          (p_TMComposition_tmp->data.decal_bkgd_color.value.rgb.green == 0.0) &&
          (p_TMComposition_tmp->data.decal_bkgd_color.value.rgb.blue == 1.0) ))
    {
	_ibmpex_stderr_print
	    ( "Defined TMComposition Entry returned incorrect data\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMComposition, 1, p_TMComposition_tmp);


    /*
     * Get the set values of an undefined entry
     */
    p_TMComposition_tmp = (PEXExtTMCompositionEntry*)PEXExtGetTableEntry(display, 
					luts[TMComposition],
					 700, PEXSetValue, &entry_type, 
					 &table_type);

    if (!((entry_type == PEXDefaultEntry) &&
	  (table_type == PEXExtLUTTMComposition) &&
          (p_TMComposition_tmp->method == PEXExtTMCompositeReplace) ))
    {
	_ibmpex_stderr_print
	    ( "Undefined TMComposition Entry returned incorrect data\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMComposition, 1, p_TMComposition_tmp);
    
    /*
     * Get a couple entries at once
     */
    status = PEXExtGetTableEntries (display, luts[TMComposition], 255, 3,  
	                         PEXSetValue, &table_type, 
				 (PEXPointer *) &p_TMComposition_tmp);

    if (!((table_type == PEXExtLUTTMComposition) &&
          (p_TMComposition_tmp[0].method == PEXExtTMCompositeDecalBackground) &&
          (p_TMComposition_tmp[0].data.decal_bkgd_color.type == 
                                                    PEXColorTypeRGB) &&
          (p_TMComposition_tmp[0].data.decal_bkgd_color.value.rgb.red == 
                                                              1.0) &&
          (p_TMComposition_tmp[0].data.decal_bkgd_color.value.rgb.green == 
                                                              0.0) &&
          (p_TMComposition_tmp[0].data.decal_bkgd_color.value.rgb.blue == 
                                                              1.0) &&
          (p_TMComposition_tmp[1].method == PEXExtTMCompositeBlendEnvColor) &&
          (p_TMComposition_tmp[1].data.blend_env.channel_number == 0 ) &&
          (p_TMComposition_tmp[1].data.blend_env.color.type == 
                                                      PEXColorTypeRGB) &&
          (p_TMComposition_tmp[1].data.blend_env.color.value.rgb.red == 
                                                                   1.0) &&
          (p_TMComposition_tmp[1].data.blend_env.color.value.rgb.green == 
                                                                   1.0) &&
          (p_TMComposition_tmp[1].data.blend_env.color.value.rgb.blue == 
                                                                   1.0) &&
          (p_TMComposition_tmp[2].method == PEXExtTMCompositeReplace) ))
    {
	_ibmpex_stderr_print
	    ( "Get TMComposition Entries returned incorrect data\n");
    }

    PEXFreeTableEntries (PEXExtLUTTMComposition, 3, p_TMComposition_tmp);


    /**********************************************************************
     * Test ends here
     *********************************************************************/
}

void inquire_test_params(char *test_name, 
			 int  *num_images, 
			 int  *supported_modes, 
			 char *win_title)
    {
    strcpy(test_name, "tmlut_edit");
    *num_images      = 1;
    *supported_modes = NON_OC_MODE;

    strcpy(win_title, "tmlut_edit");
    }

void misc_setup(Display *dpy, Window window, PEXRenderer renderer,
		int cur_image)
    {
    local_window = window;
    _ibmpex_set_stderr("tmlut_edit.stderr", "tmlut_edit");
    }

void execute_test(Display *dpy, XID resourceID, 
	       PEXOCRequestType req_type, int cur_image)
    {
    describe_test( "\nPEXlib Lookup Table Edit Tests:\n\n" );
    describe_test( "This test checks out the various aspects of editing a lookup table. It\n");
    describe_test( "tests out entry setting, entry deletion, and entry inquiry.\n");
    describe_test( "\n");
    describe_test( "The basic mechanisms for all LUTs are the same. So, the mechanisms of \n");
    describe_test( "deletion and insertion are first checked out on a TMSampling LUT. The\n");
    describe_test( "remainder of this test just sets a few entries for each type of LUT and\n");
    describe_test( "then inquires the LUTs. These tests are designed to check the set/realized\n");
    describe_test( "semantics of entry setting, check the 2 entry inquiry entry points, and check\n");
    describe_test( "that all entry fields are set correctly and returned on inquiry. \n");
    describe_test( "\n");

    lut_test(dpy, local_window);
/*
    _ibmpex_stderr_print();
*/
    }

void testproc(Display *dpy, Window window, int cur_image)
    {
    file_testproc("tmlut_edit.stderr", "tmlut_edit");
    }
